@@ -2,7 +2,7 @@ import datetime
 import logging
 
 from . import constants
-from .utils import fromTimeStamp, msgEpoch, properHex
+from .utils import fromTimeStamp, filetimeToUtc, properHex
 
 logger = logging.getLogger(__name__)
 logger.addHandler(logging.NullHandler())
@@ -15,17 +15,16 @@ def createProp(string):
     else:
         if temp not in constants.VARIABLE_LENGTH_PROPS:
             # DEBUG
-            logger.warning('Unknown property type: {}'.format(properHex(temp)))
+            logger.warning(f'Unknown property type: {properHex(temp)}')
         return VariableLengthProp(string)
 
 
-class PropBase(object):
+class PropBase:
     """
     Base class for Prop instances.
     """
 
     def __init__(self, string):
-        super(PropBase, self).__init__()
         self.__raw = string
         self.__name = properHex(string[3::-1]).upper()
         self.__type, self.__flags = constants.ST2.unpack(string)
@@ -96,9 +95,8 @@ class FixedLengthProp(PropBase):
 
     def parseType(self, _type, stream):
         """
-        Converts the data in :param stream: to a
-        much more accurate type, specified by
-        :param _type:, if possible.
+        Converts the data in :param stream: to a much more accurate type,
+        specified by :param _type:, if possible.
         :param stream: #TODO what is stream for?
 
         WARNING: Not done.
@@ -137,13 +135,13 @@ class FixedLengthProp(PropBase):
             try:
                 rawtime = constants.ST3.unpack(value)[0]
                 if rawtime != 915151392000000000:
-                    value = fromTimeStamp(msgEpoch(rawtime))
+                    value = fromTimeStamp(filetimeToUtc(rawtime))
                 else:
                     # Temporarily just set to max time to signify a null date.
                     value = datetime.datetime.max
             except Exception as e:
                 logger.exception(e)
-                logger.error('Timestamp value of {} caused an exception. This was probably caused by the time stamp being too far in the future.'.format(msgEpoch(constants.ST3.unpack(value)[0])))
+                logger.error(f'Timestamp value of {filetimeToUtc(constants.ST3.unpack(value)[0])} caused an exception. This was probably caused by the time stamp being too far in the future.')
                 logger.error(self.raw)
         elif _type == 0x0048:  # PtypGuid
             # TODO parsing for this
