@@ -9,132 +9,87 @@ import datetime
 import json
 import logging
 import logging.config
+import os
+import pathlib
 import struct
+# Not actually sure if this needs to be here for the logging, so just in case.
 import sys
+import zipfile
 
 import tzlocal
 
+from html import escape as htmlEscape
+
 from . import constants
-from .compat import os_ as os
 from .exceptions import ConversionError, IncompatibleOptionsError, InvaildPropertyIdError, UnknownCodepageError, UnknownTypeError, UnrecognizedMSGTypeError, UnsupportedMSGTypeError
 
+
 logger = logging.getLogger(__name__)
 logger.addHandler(logging.NullHandler())
 logging.addLevelName(5, 'DEVELOPER')
 
-if sys.version_info[0] >= 3:  # Python 3
-    getInput = input
 
-    makeDirs = os.makedirs
-
-    def properHex(inp, length = 0):
-        """
-        Taken (with permission) from https://github.com/TheElementalOfDestruction/creatorUtils
-        """
-        a = ''
-        if isinstance(inp, str):
-            a = ''.join([hex(ord(inp[x]))[2:].rjust(2, '0') for x in range(len(inp))])
-        elif isinstance(inp, bytes):
-            a = inp.hex()
-        elif isinstance(inp, int):
-            a = hex(inp)[2:]
-        if len(a) % 2 != 0:
-            a = '0' + a
-        return a.rjust(length, '0').upper()
-
-    def windowsUnicode(string):
-        return str(string, 'utf-16-le') if string is not None else None
-
-    from html import escape as htmlEscape
-
-else:  # Python 2
-    getInput = raw_input
-
-    def makeDirs(name, mode = 0o0777, exist_ok = False):
-        try:
-            os.makedirs(name, mode)
-        except WindowsError as e:
-            if exist_ok and e.winerror == 183: # Path exists.
-                return
-            raise
-
-    def properHex(inp, length = 0):
-        """
-        Converts the input into a hexadecimal string without the beginning "0x". The string
-        will also always have a length that is a multiple of 2 (unless :param length: has
-        been specified). :param length: only specifies the MINIMUM length that the string
-        will use.
-        """
-        a = ''
-        if isinstance(inp, (str, unicode)):
-            a = ''.join([hex(ord(inp[x]))[2:].rjust(2, '0') for x in range(len(inp))])
-        elif isinstance(inp, int):
-            a = hex(inp)[2:]
-        elif isinstance(inp, long):
-            a = hex(inp)[2:-1]
-        if len(a) % 2 != 0:
-            a = '0' + a
-        return a.rjust(length, '0').upper()
-
-    def windowsUnicode(string):
-        return unicode(string, 'utf-16-le') if string is not None else None
-
-    from cgi import escape as htmlEscape
-
-def addNumToDir(dirName):
+def addNumToDir(dirName : pathlib.Path) -> pathlib.Path:
     """
     Attempt to create the directory with a '(n)' appended.
     """
     for i in range(2, 100):
         try:
-            newDirName = dirName + ' (' + str(i) + ')'
-            makeDirs(newDirName)
+            newDirName = dirName.with_name(dirName.name + f' ({i})')
+            os.makedirs(newDirName)
             return newDirName
         except Exception as e:
             pass
     return None
 
-def addNumToZipDir(dirName, _zip):
+def addNumToZipDir(dirName : pathlib.Path, _zip):
     """
     Attempt to create the directory with a '(n)' appended.
     """
     for i in range(2, 100):
-        newDirName = dirName + ' (' + str(i) + ')'
-        if not any(x.startswith(newDirName.rstrip('/') + '/') for x in _zip.namelist()):
+        newDirName = dirName.with_name(dirName.name + f' ({i})')
+        pathCompare = str(newDirName).rstrip('/') + '/'
+        if not any(x.startswith(pathCompare) for x in _zip.namelist()):
             return newDirName
     return None
 
-def bitwiseAdjust(inp, mask):
+def bitwiseAdjust(inp : int, mask : int) -> int:
     """
     Uses a given mask to adjust the location of bits after an operation like
     bitwise AND. This is useful for things like flags where you are trying to
     get a small portion of a larger number. Say for example, you had the number
     0xED (0b11101101) and you needed the adjusted result of the AND operation
-    with 0x70 (0b01110000). The result of the and operation (0b01100000) and the
-    mask used to get it (0x70) are give and the output gets adjusted to be 0x6
-    (0b110).
+    with 0x70 (0b01110000). The result of the AND operation (0b01100000) and the
+    mask used to get it (0x70) are given to this function and the adjustment
+    will be done automatically.
 
     :param mask: MUST be greater than 0.
+
+    :raises ValueError: if the mask is not greater than 0.
     """
     if mask < 1:
         raise ValueError('Mask MUST be greater than 0')
     return inp >> bin(mask)[::-1].index('1')
 
-def bitwiseAdjustedAnd(inp, mask):
+def bitwiseAdjustedAnd(inp : int, mask : int) -> int:
     """
     Preforms the bitwise AND operation between :param inp: and :param mask: and
     adjusts the results based on the rules of the bitwiseAdjust function.
+
+    :raises ValueError: if the mask is not greater than 0.
     """
     if mask < 1:
         raise ValueError('Mask MUST be greater than 0')
     return (inp & mask) >> bin(mask)[::-1].index('1')
 
-def bytesToGuid(bytes_input):
-    hexinput = [properHex(byte) for byte in bytes_input]
-    hexs = [hexinput[3] + hexinput[2] + hexinput[1] + hexinput[0], hexinput[5] + hexinput[4], hexinput[7] + hexinput[6], hexinput[8] + hexinput[9], ''.join(hexinput[10:16])]
-    return '{{{}-{}-{}-{}-{}}}'.format(*hexs).upper()
+def bytesToGuid(bytesInput : bytes) -> str:
+    """
+    Converts a bytes instance to a GUID.
+    """
+    guidVals = constants.ST_GUID.unpack(bytesInput)
+    return f'{{{guidVals[0]:08X}-{guidVals[1]:04X}-{guidVals[2]:04X}-{guidVals[3][:2].hex().upper()}-{guidVals[3][2:].hex().upper()}}}'
 
-def ceilDiv(n, d):
+def ceilDiv(n : int, d : int) -> int:
     """
     Returns the int from the ceil division of n / d.
     ONLY use ints as inputs to this function.
@@ -144,14 +99,24 @@ def ceilDiv(n, d):
     """
     return -(n // -d)
 
-def divide(string, length):
+def createZipOpen(func):
     """
-    Taken (with permission) from https://github.com/TheElementalOfDestruction/creatorUtils
+    Creates a wrapper for the open function of a ZipFile that will automatically
+    set the current date as the modified time to the current time.
+    """
+    def _open(name, mode, *args, **kwargs):
+        if mode == 'w':
+            name = zipfile.ZipInfo(name, datetime.datetime.now().timetuple())
 
-    Divides a string into multiple substrings of equal length.
-    If there is not enough for the last substring to be equal,
-    it will simply use the rest of the string.
-    Can also be used for things like lists and tuples.
+        return func(name, mode, *args, **kwargs)
+
+    return _open
+
+def divide(string, length : int) -> list:
+    """
+    Divides a string into multiple substrings of equal length. If there is not
+    enough for the last substring to be equal, it will simply use the rest of
+    the string. Can also be used for things like lists and tuples.
 
     :param string: string to be divided.
     :param length: length of each division.
@@ -167,14 +132,20 @@ def divide(string, length):
     """
     return [string[length * x:length * (x + 1)] for x in range(int(ceilDiv(len(string), length)))]
 
-def fromTimeStamp(stamp):
+def fromTimeStamp(stamp) -> datetime.datetime:
+    """
+    Returns a datetime from the UTC timestamp given the current timezone.
+    """
     return datetime.datetime.fromtimestamp(stamp, tzlocal.get_localzone())
 
 def getCommandArgs(args):
     """
-    Parse command-line arguments
+    Parse command-line arguments.
+
+    :raises IncompatibleOptionsError: if options are provided that are
+        incompatible.
     """
-    parser = argparse.ArgumentParser(description=constants.MAINDOC, prog='extract_msg')
+    parser = argparse.ArgumentParser(description = constants.MAINDOC, prog = 'extract_msg')
     # --use-content-id, --cid
     parser.add_argument('--use-content-id', '--cid', dest='cid', action='store_true',
                         help='Save attachments by their Content ID, if they have one. Useful when working with the HTML body.')
@@ -210,27 +181,33 @@ def getCommandArgs(args):
                         help='Tells the program to dump the message body (plain text) to stdout. Overrides saving arguments.')
     # --html
     parser.add_argument('--html', dest='html', action='store_true',
-                       help='Sets whether the output should be html. If this is not possible, will error.')
+                        help='Sets whether the output should be HTML. If this is not possible, will error.')
+    # --prepared-html
+    parser.add_argument('--prepared-html', dest='preparedHtml', action='store_true',
+                        help='When used in conjunction with --html, sets whether the HTML output should be prepared for embedded attachments.')
     # --raw
     parser.add_argument('--raw', dest='raw', action='store_true',
-                       help='Sets whether the output should be html. If this is not possible, will error.')
+                        help='Sets whether the output should be HTML. If this is not possible, will error.')
     # --rtf
     parser.add_argument('--rtf', dest='rtf', action='store_true',
-                       help='Sets whether the output should be rtf. If this is not possible, will error.')
+                        help='Sets whether the output should be RTF. If this is not possible, will error.')
     # --allow-fallback
     parser.add_argument('--allow-fallback', dest='allowFallback', action='store_true',
-                       help='Tells the program to fallback to a different save type if the selected one is not possible.')
+                        help='Tells the program to fallback to a different save type if the selected one is not possible.')
+    # --zip
+    parser.add_argument('--zip', dest='zip',
+                        help='Path to use for saving to a zip file.')
     # --out-name NAME
-    parser.add_argument('--out-name', dest = 'out_name',
-                        help = 'Name to be used with saving the file output. Should come immediately after the file name.')
+    parser.add_argument('--out-name', dest='out_name',
+                        help='Name to be used with saving the file output. Should come immediately after the file name.')
     # [msg files]
     parser.add_argument('msgs', metavar='msg', nargs='+',
-                        help='An msg file to be parsed')
+                        help='An msg file to be parsed.')
 
     options = parser.parse_args(args)
 
     # Check if more than one of the following arguments has been specified
-    if options.html + options.rtf + options.json > 1:
+    if options.html + options.rtf + options.json + options.raw > 1:
        raise IncompatibleOptionsError('Only one of these options may be selected at a time: --html, --json, --raw, --rtf')
 
     if options.dev or options.file_logging:
@@ -269,15 +246,12 @@ def getCommandArgs(args):
     options.msgs = file_tables
     return options
 
-def getContFileDir(_file_):
-    """
-    Takes in the path to a file and tries to return the containing folder.
-    """
-    return '/'.join(_file_.replace('\\', '/').split('/')[:-1])
-
-def getEncodingName(codepage):
+def getEncodingName(codepage : int) -> str:
     """
     Returns the name of the encoding with the specified codepage.
+
+    :raises UnknownCodepageError: if the codepage is unrecognized.
+    :raises UnsupportedEncodingError: if the codepage is not supported.
     """
     if codepage not in constants.CODE_PAGES:
         raise UnknownCodepageError(str(codepage))
@@ -285,25 +259,44 @@ def getEncodingName(codepage):
         codecs.lookup(constants.CODE_PAGES[codepage])
         return constants.CODE_PAGES[codepage]
     except LookupError:
-        raise UnsupportedEncodingError('The codepage {} ({}) is not currently supported by your version of Python.'.format(codepage, constants.CODE_PAGES[codepage]))
+        raise UnsupportedEncodingError(f'The codepage {codepage} ({constants.CODE_PAGES[codepage]}) is not currently supported by your version of Python.')
 
 def getFullClassName(inp):
     return inp.__class__.__module__ + '.' + inp.__class__.__name__
 
-def hasLen(obj):
+def hasLen(obj) -> bool:
     """
     Checks if :param obj: has a __len__ attribute.
     """
     return hasattr(obj, '__len__')
 
-def injectHtmlHeader(msgFile):
+def injectHtmlHeader(msgFile, prepared : bool = False) -> bytes:
     """
     Returns the HTML body from the MSG file (will check that it has one) with
     the HTML header injected into it.
+
+    :param prepared: Determines whether to be using the standard HTML (False) or
+                     the prepared HTML (True) body (Default: False).
+
+    :raises AttributeError: if the correct HTML body cannot be acquired.
     """
     if not hasattr(msgFile, 'htmlBody') or not msgFile.htmlBody:
         raise AttributeError('Cannot inject the HTML header without an HTML body attribute.')
 
+    body = None
+
+    # We don't do this all at once because the prepared body is not cached.
+    if prepared:
+        if hasattr(msgFile, 'htmlBodyPrepared'):
+            body = msgFile.htmlBodyPrepared
+
+        # If the body is not valid or not found, raise an AttributeError.
+        if not body:
+            raise AttributeError('Cannot find a prepared HTML body to inject into.')
+
+    else:
+        body = msgFile.htmlBody
+
     def replace(bodyMarker):
         """
         Internal function to replace the body tag with itself plus the header.
@@ -319,12 +312,15 @@ def injectHtmlHeader(msgFile):
         }).encode('utf-8')
 
     # Use the previously defined function to inject the HTML header.
-    return constants.RE_HTML_BODY_START.sub(replace, msgFile.htmlBody, 1)
+    return constants.RE_HTML_BODY_START.sub(replace, body, 1)
 
-def injectRtfHeader(msgFile):
+def injectRtfHeader(msgFile) -> bytes:
     """
     Returns the RTF body from the MSG file (will check that it has one) with the
     RTF header injected into it.
+
+    :raises AttributeError: if the RTF body cannot be acquired.
+    :raises RuntimeError: if all injection attempts fail.
     """
     if not hasattr(msgFile, 'rtfBody') or not msgFile.rtfBody:
         raise AttributeError('Cannot inject the RTF header without an RTF body attribute.')
@@ -396,19 +392,24 @@ def injectRtfHeader(msgFile):
         logger.debug('Successfully injected RTF header using sixth method.')
         return data
 
-    raise Exception('All injection attempts failed.')
+    raise RuntimeError('All injection attempts failed. Please report this to the developer.')
 
-def inputToBytes(stringInputVar, encoding):
-    if isinstance(stringInputVar, constants.BYTES):
+def inputToBytes(stringInputVar, encoding) -> bytes:
+    """
+    Converts the input into bytes.
+
+    :raises ConversionError: if the input cannot be converted.
+    """
+    if isinstance(stringInputVar, bytes):
         return stringInputVar
-    elif isinstance(stringInputVar, constants.STRING):
+    elif isinstance(stringInputVar, str):
         return stringInputVar.encode(encoding)
     elif stringInputVar is None:
         return b''
     else:
-        raise ConversionError('Cannot convert to BYTES type')
+        raise ConversionError('Cannot convert to bytes.')
 
-def inputToMsgpath(inp):
+def inputToMsgpath(inp) -> list:
     """
     Converts the input into an msg path.
     """
@@ -417,17 +418,22 @@ def inputToMsgpath(inp):
     ret = inputToString(inp, 'utf-8').replace('\\', '/').split('/')
     return ret if ret[0] != '' else []
 
-def inputToString(bytesInputVar, encoding):
-    if isinstance(bytesInputVar, constants.STRING):
+def inputToString(bytesInputVar, encoding) -> str:
+    """
+    Converts the input into a string.
+
+    :raises ConversionError: if the input cannot be converted.
+    """
+    if isinstance(bytesInputVar, str):
         return bytesInputVar
-    elif isinstance(bytesInputVar, constants.BYTES):
+    elif isinstance(bytesInputVar, bytes):
         return bytesInputVar.decode(encoding)
     elif bytesInputVar is None:
         return ''
     else:
-        raise ConversionError('Cannot convert to STRING type')
+        raise ConversionError('Cannot convert to str type.')
 
-def isEncapsulatedRtf(inp):
+def isEncapsulatedRtf(inp : bytes) -> bool:
     """
     Currently the destection is made to be *extremly* basic, but this will work
     for now. In the future this will be fixed to that literal text in the body
@@ -435,13 +441,13 @@ def isEncapsulatedRtf(inp):
     """
     return b'\\fromhtml' in inp
 
-def isEmptyString(inp):
+def isEmptyString(inp : str) -> bool:
     """
     Returns true if the input is None or is an Empty string.
     """
     return (inp == '' or inp is None)
 
-def knownMsgClass(classType):
+def knownMsgClass(classType : str) -> bool:
     """
     Checks if the specified class type is recognized by the module. Usually used
     for checking if a type is simply unsupported rather than unknown.
@@ -451,20 +457,21 @@ def knownMsgClass(classType):
         return True
 
     for item in constants.KNOWN_CLASS_TYPES:
-        if classType.startswith(item):
+        if classType.startsWith(item):
             return True
 
     return False
 
-def msgEpoch(inp):
+def filetimeToUtc(inp : int) -> float:
     """
-    Taken (with permission) from https://github.com/TheElementalOfDestruction/creatorUtils
+    Converts a FILETIME into a unix timestamp.
     """
     return (inp - 116444736000000000) / 10000000.0
 
-def msgpathToString(inp):
+def msgpathToString(inp) -> str:
     """
-    Converts an msgpath (one of the internal paths inside an msg file) into a string.
+    Converts an msgpath (one of the internal paths inside an msg file) into a
+    string.
     """
     if inp is None:
         return None
@@ -478,13 +485,11 @@ def openMsg(path, prefix = '', attachmentClass = None, filename = None, delayAtt
     Function to automatically open an MSG file and detect what type it is.
 
     :param path: Path to the msg file in the system or is the raw msg file.
-    :param prefix: Used for extracting embeded msg files
-        inside the main one. Do not set manually unless
-        you know what you are doing.
-    :param attachmentClass: Optional, the class the Message object
-        will use for attachments. You probably should
-        not change this value unless you know what you
-        are doing.
+    :param prefix: Used for extracting embeded msg files inside the main one.
+        Do not set manually unless you know what you are doing.
+    :param attachmentClass: Optional, the class the Message object will use for
+        attachments. You probably should not change this value unless you know
+        what you are doing.
     :param filename: Optional, the filename to be used by default when saving.
     :param delayAttachments: Optional, delays the initialization of attachments
         until the user attempts to retrieve them. Allows MSG files with bad
@@ -500,7 +505,8 @@ def openMsg(path, prefix = '', attachmentClass = None, filename = None, delayAtt
     when it cannot identify what MSGFile derivitive to use. Otherwise, it will
     log the error and return a basic MSGFile instance.
 
-    Raises UnsupportedMSGTypeError and UnrecognizedMSGTypeError.
+    :raises UnsupportedMSGTypeError: if the type is recognized but not suppoted.
+    :raises UnrecognizedMSGTypeError: if the type is not recognized.
     """
     from .appointment import Appointment
     from .attachment import Attachment
@@ -525,26 +531,29 @@ def openMsg(path, prefix = '', attachmentClass = None, filename = None, delayAtt
     elif classType == 'ipm': # Unspecified format. It should be equal to this and not just start with it.
         return msg
     elif strict:
+        # Because we are closing it, we need to store it in a variable first.
         ct = msg.classType
         msg.close()
         if knownMsgClass(classType):
-            raise UnsupportedMSGTypeError('MSG type "{}" currently is not supported by the module. If you would like support, please make a feature request.'.format(ct))
-        raise UnrecognizedMSGTypeError('Could not recognize msg class type "{}".'.format(ct))
+            raise UnsupportedMSGTypeError(f'MSG type "{ct}" currently is not supported by the module. If you would like support, please make a feature request.')
+        raise UnrecognizedMSGTypeError(f'Could not recognize msg class type "{ct}".')
     else:
-        logger.error('Could not recognize msg class type "{}". This most likely means it hasn\'t been implemented yet, and you should ask the developers to add support for it.'.format(msg.classType))
+        logger.error(f'Could not recognize msg class type "{msg.classType}". This most likely means it hasn\'t been implemented yet, and you should ask the developers to add support for it.')
         return msg
 
-def parseType(_type, stream, encoding, extras):
+def parseType(_type : int, stream, encoding, extras):
     """
-    Converts the data in :param stream: to a
-    much more accurate type, specified by
+    Converts the data in :param stream: to a much more accurate type, specified
+    by :param _type:.
     :param _type: the data's type.
     :param stream: is the data to be converted.
     :param encoding: is the encoding to be used for regular strings.
     :param extras: is used in the case of types like PtypMultipleString.
-    For that example, extras should be a list of the bytes from rest of the streams.
+    For that example, extras should be a list of the bytes from rest of the
+    streams.
 
-    WARNING: Not done. Do not try to implement anywhere where it is not already implemented
+    :raises NotImplementedError: for types with no current support. Most of
+        these types have no documentation of existing in an MSG file.
     """
     # WARNING Not done. Do not try to implement anywhere where it is not already implemented.
     value = stream
@@ -572,13 +581,16 @@ def parseType(_type, stream, encoding, extras):
     elif _type == 0x000A:  # PtypErrorCode
         value = constants.STUI32.unpack(value)[0]
         # TODO parsing for this.
-        # I can't actually find any msg properties that use this, so it should be okay to release this function without support for it.
-        raise NotImplementedError('Parsing for type 0x000A has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType 0x000A"')
+        # I can't actually find any msg properties that use this, so it should
+        # be okay to release this function without support for it.
+        raise NotImplementedError('Parsing for type 0x000A (PtypErrorCode) has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType 0x000A PtypErrorCode".')
     elif _type == 0x000B:  # PtypBoolean
         return constants.ST3.unpack(value)[0] == 1
     elif _type == 0x000D:  # PtypObject/PtypEmbeddedTable
         # TODO parsing for this
-        # Wait, that's the extension for an attachment folder, so parsing this might not be as easy as we would hope. The function may be released without support for this.
+        # Wait, that's the extension for an attachment folder, so parsing this
+        # might not be as easy as we would hope. The function may be released
+        # without support for this.
         raise NotImplementedError('Current version of extract-msg does not support the parsing of PtypObject/PtypEmbeddedTable in this function.')
     elif _type == 0x0014:  # PtypInteger64
         return constants.STI64.unpack(value)[0]
@@ -589,7 +601,7 @@ def parseType(_type, stream, encoding, extras):
     elif _type == 0x0040:  # PtypTime
         rawtime = constants.ST3.unpack(value)[0]
         if rawtime != 915151392000000000:
-            value = fromTimeStamp(msgEpoch(rawtime))
+            value = fromTimeStamp(filetimeToUtc(rawtime))
         else:
             # Temporarily just set to max time to signify a null date.
             value = datetime.datetime.max
@@ -597,63 +609,68 @@ def parseType(_type, stream, encoding, extras):
     elif _type == 0x0048:  # PtypGuid
         return bytesToGuid(value)
     elif _type == 0x00FB:  # PtypServerId
-        # TODO parsing for this
-        raise NotImplementedError('Parsing for type 0x00FB has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType 0x00FB"')
+        count = constants.STUI16.unpack(value[:2])
+        # If the first byte is a 1 then it uses the ServerID structure.
+        if value[3] == 1:
+            from .data import ServerID
+            return ServerID(value)
+        else:
+            return (count, value[2:count + 2])
     elif _type == 0x00FD:  # PtypRestriction
         # TODO parsing for this
-        raise NotImplementedError('Parsing for type 0x00FD has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType 0x00FD"')
+        raise NotImplementedError('Parsing for type 0x00FD (PtypRestriction) has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType 0x00FD PtypRestriction".')
     elif _type == 0x00FE:  # PtypRuleAction
         # TODO parsing for this
-        raise NotImplementedError('Parsing for type 0x00FE has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType 0x00FE"')
+        raise NotImplementedError('Parsing for type 0x00FE (PtypRuleAction) has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType 0x00FE PtypRuleAction".')
     elif _type == 0x0102:  # PtypBinary
         return value
     elif _type & 0x1000 == 0x1000:  # PtypMultiple
         # TODO parsing for `multiple` types
-        if _type in (0x101F, 0x101E):
+        if _type in (0x101F, 0x101E): # PtypMultipleString/PtypMultipleString8
             ret = [x.decode(encoding) for x in extras]
-            lengths = struct.unpack('<{}i'.format(len(ret)), stream)
+            lengths = struct.unpack(f'<{len(ret)}i', stream)
             lengthLengths = len(lengths)
             if lengthLengths > lengthExtras:
-                logger.warning('Error while parsing multiple type. Expected {} stream{}, got {}. Ignoring.'.format(lengthLengths, 's' if lengthLengths != 1 else '', lengthExtras))
+                logger.warning(f'Error while parsing multiple type. Expected {lengthLengths} stream{"s" if lengthLengths != 1 else ""}, got {lengthExtras}. Ignoring.')
             for x, y in enumerate(extras):
                 if lengths[x] != len(y):
-                    logger.warning('Error while parsing multiple type. Expected length {}, got {}. Ignoring.'.format(lengths[x], len(y)))
+                    logger.warning(f'Error while parsing multiple type. Expected length {lengths[x]}, got {len(y)}. Ignoring.')
             return ret
-        elif _type == 0x1102:
+        elif _type == 0x1102: # PtypMultipleBinary
             ret = copy.deepcopy(extras)
             lengths = tuple(constants.STUI32.unpack(stream[pos*8:(pos+1)*8])[0] for pos in range(len(stream) // 8))
             lengthLengths = len(lengths)
             if lengthLengths > lengthExtras:
-                logger.warning('Error while parsing multiple type. Expected {} stream{}, got {}. Ignoring.'.format(lengthLengths, 's' if lengthLengths != 1 else '', lengthExtras))
+                logger.warning(f'Error while parsing multiple type. Expected {lengthLengths} stream{"s" if lengthLengths != 1 else ""}, got {lengthExtras}. Ignoring.')
             for x, y in enumerate(extras):
                 if lengths[x] != len(y):
-                    logger.warning('Error while parsing multiple type. Expected length {}, got {}. Ignoring.'.format(lengths[x], len(y)))
+                    logger.warning(f'Error while parsing multiple type. Expected length {lengths[x]}, got {len(y)}. Ignoring.')
             return ret
         elif _type in (0x1002, 0x1003, 0x1004, 0x1005, 0x1007, 0x1014, 0x1040, 0x1048):
             if stream != len(extras):
-                logger.warning('Error while parsing multiple type. Expected {} entr{}, got {}. Ignoring.'.format(stream, ('y' if stream == 1 else 'ies'), len(extras)))
-            if _type == 0x1002:
+                logger.warning(f'Error while parsing multiple type. Expected {stream} entr{"y" if stream == 1 else "ies"}, got {len(extras)}. Ignoring.')
+            if _type == 0x1002: # PtypMultipleInteger16
                 return tuple(constants.STMI16.unpack(x)[0] for x in extras)
-            if _type == 0x1003:
+            if _type == 0x1003: # PtypMultipleInteger32
                 return tuple(constants.STMI32.unpack(x)[0] for x in extras)
-            if _type == 0x1004:
+            if _type == 0x1004: # PtypMultipleFloating32
                 return tuple(constants.STMF32.unpack(x)[0] for x in extras)
-            if _type == 0x1005:
+            if _type == 0x1005: # PtypMultipleFloating64
                 return tuple(constants.STMF64.unpack(x)[0] for x in extras)
-            if _type == 0x1007:
+            if _type == 0x1007: # PtypMultipleFloatingTime
                 values = tuple(constants.STMF64.unpack(x)[0] for x in extras)
-                raise NotImplementedError('Parsing for type 0x1007 has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType 0x1007"')
-            if _type == 0x1014:
+                return tuple(constants.PYTPFLOATINGTIME_START + datetime.timedelta(days = amount) for amount in values)
+            if _type == 0x1014: # PtypMultipleInteger64
                 return tuple(constants.STMI64.unpack(x)[0] for x in extras)
-            if _type == 0x1040:
-                return tuple(msgEpoch(constants.ST3.unpack(x)[0]) for x in extras)
-            if _type == 0x1048:
+            if _type == 0x1040: # PtypMultipleTime
+                return tuple(filetimeToUtc(constants.ST3.unpack(x)[0]) for x in extras)
+            if _type == 0x1048: # PtypMultipleGuid
                 return tuple(bytesToGuid(x) for x in extras)
         else:
-            raise NotImplementedError('Parsing for type {} has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType {}"'.format(_type, _type))
+            raise NotImplementedError(f'Parsing for type {_type} has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType {_type}".')
     return value
 
-def prepareFilename(filename):
+def prepareFilename(filename) -> str:
     """
     Adjusts :param filename: so that it can succesfully be used as an actual
     file name.
@@ -661,13 +678,29 @@ def prepareFilename(filename):
     # I would use re here, but it tested to be slightly slower than this.
     return ''.join(i for i in filename if i not in r'\/:*?"<>|' + '\x00')
 
-def roundUp(inp, mult):
+def properHex(inp, length : int = 0) -> str:
+    """
+    Takes in various input types and converts them into a hex string whose
+    length will always be even.
+    """
+    a = ''
+    if isinstance(inp, str):
+        a = ''.join([hex(ord(inp[x]))[2:].rjust(2, '0') for x in range(len(inp))])
+    elif isinstance(inp, bytes):
+        a = inp.hex()
+    elif isinstance(inp, int):
+        a = hex(inp)[2:]
+    if len(a) % 2 != 0:
+        a = '0' + a
+    return a.rjust(length, '0').upper()
+
+def roundUp(inp : int, mult : int) -> int:
     """
     Rounds :param inp: up to the nearest multiple of :param mult:.
     """
     return inp + (mult - inp) % mult
 
-def rtfSanitizeHtml(inp):
+def rtfSanitizeHtml(inp : str) -> str:
     """
     Sanitizes input to an RTF stream that has encapsulated HTML.
     """
@@ -697,7 +730,10 @@ def rtfSanitizeHtml(inp):
 
     return output
 
-def rtfSanitizePlain(inp):
+def rtfSanitizePlain(inp : str) -> str:
+    """
+    Sanitizes input to a plain RTF stream.
+    """
     if not inp:
         return ''
     output = ''
@@ -716,58 +752,59 @@ def rtfSanitizePlain(inp):
 
     return output
 
-def setupLogging(defaultPath=None, defaultLevel=logging.WARN, logfile=None, enableFileLogging=False,
-                  env_key='EXTRACT_MSG_LOG_CFG'):
+def setupLogging(defaultPath = None, defaultLevel = logging.WARN, logfile = None, enableFileLogging : bool = False,
+                  env_key = 'EXTRACT_MSG_LOG_CFG') -> bool:
     """
     Setup logging configuration
 
     Args:
-        defaultPath (str): Default path to use for the logging configuration file
-        defaultLevel (int): Default logging level
-        env_key (str): Environment variable name to search for, for setting logfile path
+    :param defaultPath: Default path to use for the logging configuration file.
+    :param defaultLevel: Default logging level.
+    :param env_key: Environment variable name to search for, for setting logfile
+        path.
+    :param enableFileLogging: Whether to use a file to log or not.
 
     Returns:
         bool: True if the configuration file was found and applied, False otherwise
     """
-    shippedConfig = getContFileDir(__file__) + '/logging-config/'
+    shippedConfig = pathlib.Path(__file__).parent / 'logging-config'
     if os.name == 'nt':
         null = 'NUL'
-        shippedConfig += 'logging-nt.json'
+        shippedConfig /= 'logging-nt.json'
     elif os.name == 'posix':
         null = '/dev/null'
-        shippedConfig += 'logging-posix.json'
+        shippedConfig /= 'logging-posix.json'
     # Find logging.json if not provided
-    if not defaultPath:
-        defaultPath = shippedConfig
+    defaultPath = pathlib.Path(defaultPath) if defaultPath else shippedConfig
 
     paths = [
         defaultPath,
-        'logging.json',
-        '../logging.json',
-        '../../logging.json',
+        pathlib.Path('logging.json'),
+        pathlib.Path('../logging.json'),
+        pathlib.Path('../../logging.json'),
         shippedConfig,
     ]
 
     path = None
 
     for configPath in paths:
-        if os.path.exists(configPath):
+        if configPath.exists():
             path = configPath
             break
 
     value = os.getenv(env_key, None)
-    if value and os.path.exists(value):
-        path = value
+    if value and os.path.exists(value) and os.path.isfile(value):
+        path = pathlib.Path(value)
 
-    if path is None:
+    if not path:
         print('Unable to find logging.json configuration file')
         print('Make sure a valid logging configuration file is referenced in the defaultPath'
               ' argument, is inside the extract_msg install location, or is available at one '
               'of the following file-paths:')
         print(str(paths[1:]))
-        logging.basicConfig(level=defaultLevel)
+        logging.basicConfig(level = defaultLevel)
         logging.warning('The extract_msg logging configuration was not found - using a basic configuration.'
-                        'Please check the extract_msg installation directory for "logging-{}.json".'.format(os.name))
+                        f'Please check the extract_msg installation directory for "logging-{os.name}.json".')
         return False
 
     with open(path, 'rt') as f:
@@ -778,9 +815,9 @@ def setupLogging(defaultPath=None, defaultLevel=logging.WARN, logfile=None, enab
             if enableFileLogging:
                 config['handlers'][x]['filename'] = tmp = os.path.expanduser(
                     os.path.expandvars(logfile if logfile else config['handlers'][x]['filename']))
-                tmp = getContFileDir(tmp)
-                if not os.path.exists(tmp):
-                    makeDirs(tmp)
+                tmp = pathlib.Path(tmp).parent
+                if not tmp.exists:
+                    os.makedirs(tmp)
             else:
                 config['handlers'][x]['filename'] = null
 
@@ -793,10 +830,14 @@ def setupLogging(defaultPath=None, defaultLevel=logging.WARN, logfile=None, enab
     logging.getLogger().setLevel(defaultLevel)
     return True
 
-def verifyPropertyId(id):
+def verifyPropertyId(id : str) -> None:
     """
-    Determines whether a property ID is valid for the functions that this function
-    is called from. Property IDs MUST be a 4 digit hexadecimal string.
+    Determines whether a property ID is valid for vertain functions. Property
+    IDs MUST be a 4 digit hexadecimal string. Property is valid if no exception
+    is raised.
+
+    :raises InvaildPropertyIdError: if the it is not a 4 digit hexadecimal
+        number.
     """
     if not isinstance(id, str):
         raise InvaildPropertyIdError('ID was not a 4 digit hexadecimal string')
@@ -809,6 +850,14 @@ def verifyPropertyId(id):
             raise InvaildPropertyIdError('ID was not a 4 digit hexadecimal string')
 
 def verifyType(_type):
+    """
+    Verifies that the type is valid. Raises an exception if it is not.
+
+    :raises UnknownTypeError: if the type is not recognized.
+    """
     if _type is not None:
         if (_type not in constants.VARIABLE_LENGTH_PROPS_STRING) and (_type not in constants.FIXED_LENGTH_PROPS_STRING):
-            raise UnknownTypeError('Unknown type {}'.format(_type))
+            raise UnknownTypeError(f'Unknown type {_type}.')
+
+def windowsUnicode(string):
+    return str(string, 'utf-16-le') if string is not None else None
